# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# Written by Tim-Philipp Müller <tim centricular net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from codeina import check, import_ui_string
from codeina.fxml import XMLObject
from codeina.extern.log import log

class ProductList(XMLObject):
    products = None
    url = None
    id = None
    logo = None

    def dbg_print_part_products(self):
        products_parts_str = ""
        for p in self.products:
            for p2 in self.products:
                if p.is_part_product_of(p2):
                    products_parts_str += "Product '%s' is part of product '%s'\n" % (p.name,
                            p2.name)
        return products_parts_str

    def count_selected(self):
        num = 0
        for p in self.products:
            if p.selected:
                num += 1
        return num

    def get_selected(self):
        sel = []
        for p in self.products:
            if p.selected:
                sel.append(p)
        return sel

    def has_products_for_current_system (self):
        for product in self.products:
            if product.get_flavors_for_current_system ():
                return True
        return False

    def filter_products_for_current_system (self):
        products_current = []
        for product in self.products:
            if product.get_flavors_for_current_system ():
                products_current.append (product)
        self.products = products_current
        return len( self.products) > 0

    def get_intersection (self, product_list):
        new_product_list = ProductList ()
        new_product_list.id = self.id
        new_product_list.url = self.url
        new_product_list.logo = self.logo
        new_product_list.products = []
        for product in product_list:
            if product in self.products:
                new_product_list.products.append (product)
        return new_product_list

    def product_length (self):
        return len (self.products)

    def get_logo_filename (self):
        from codeina import configure
        import os
        path = None
        if self.logo and os.path.exists (self.logo):
            return self.logo
        if self.id == 'fluendo': 
            return configure.make_logo_path ('fluendo.png')
        elif self.logo:
            path = configure.make_logo_path (self.logo + '.png')
        else:
            path = configure.make_logo_path (self.id + '.png')
            if os.path.exists(path): return path
        # if the image can not be found use a cdrom image
        return configure.make_logo_path ('media-optical.png')

    def get_logo_pixbuf (self, square_size):
        import gtk
        pixbuf = gtk.gdk.pixbuf_new_from_file (self.get_logo_filename ())
        w = float (pixbuf.get_width ())
        h = float (pixbuf.get_height ())
        if w > h:
            h = ( h / w ) * square_size
            w = square_size
        else:
            w = ( w / h ) * square_size
            h = square_size
        return pixbuf.scale_simple (int (w), int (h), gtk.gdk.INTERP_BILINEAR)

    def mark_all_as_legal (self, value):
        for p in self.products:
            p.mark_as_legal (value)

    def check_legal (self):
        if self.id == 'fluendo':
            log.log ('restricted', 'marking fluendo products as legal')
            self.mark_all_as_legal (True)
        else:
            for p in self.products:
                p.check_legal ()

class ElementType (XMLObject):
    """
    Describe a element type like decoders or urisources
    """
    caps = None

    def __repr__ (self):
        return self.caps

    def check_legal (self):
        from codeina import restrictedproducts
        import gst
        my_caps = gst.caps_from_string(self.caps)
        for res_string in restrictedproducts.restricted_caps:
            if self.caps == res_string:
                log.log ('restricted', 'found restricted caps %s' % my_caps)
                return False
            other_caps = gst.caps_from_string (res_string)
            intersect = my_caps.intersect(other_caps)
            if intersect.get_size() > 0:
                log.log ('restricted', 'found restricted caps %s' % my_caps)
                return False
        return True

class Element (XMLObject):
    """
    Describes the element in the plugins
    """
    name = None
    decoders = None
    urisources = None

    def __repr__ (self):
        return self.name

    def decoder_count (self):
        return len (self.decoders)

    def urisources_count (self):
        return len (self.urisources)

    def check_legal (self):
        plugins = []
        if self.decoders:
            plugins += self.decoders
        if self.urisources:
            plugins += self.urisources
        for i in plugins:
            if not i.check_legal ():
                return False
        return True

class Plugin (XMLObject):
    """
    Describes the plugin in a product
    """
    name = None
    description = None
    version = None
    origin = None
    elements = None

    #cache for installed status
    cache_is_newer = None
    cache_is_installed = None

    def element_count (self):
        return len (self.elements)

    def __repr__ (self):
        return self.name

    def check_legal (self):
        from codeina import restrictedproducts
        if self.name in restrictedproducts.free_plugins:
            log.log ('restricted', 'is marked as a legal plugin (%s)' % self.name)
            return True
        # Check if the caps are restricted
        for i in self.elements:
            if not i.check_legal ():
                log.log ('restricted', 'is has restricted caps (%s)' % self.name)
                return False
        log.log ('restricted', 'no restricted caps found on this element (%s)' % self.name)
        return True

    def is_installed (self):
        if self.cache_is_installed != None:
            return self.cache_is_installed
        local_version = check.get_plugin_version(self.name)
        self.cache_is_installed = False
        if not local_version:
            return False
        self.cache_is_installed = True
        return True

    def is_newer_than_installed (self):
        if self.cache_is_newer != None:
            return self.cache_is_newer
        local_version = check.get_plugin_version(self.name)
        self.cache_is_newer = False
        if not local_version or not self.version:
            return False
        my_version = check.Version(self.version)
        log.debug ('codeina', 'installed version (%s), available version (%s)' % (local_version, my_version))
        if my_version.is_newer_than (local_version):
            self.cache_is_newer = True
            return True
        return False

    def flush_cache (self):
        self.cache_is_newer = None
        self.cache_is_installed = None

class Product(XMLObject):
    """
    Describes a product that can be bought.
    """
    name = None
    rank = 0
    price = 0
    logo_uri = None
    eula_uri = None
    flavors = None
    summary = None
    plugins = None
    legal = True

    cache_installed_status = None
    cache_updates_available = None

    selected = False # for list view

    def __repr__(self):
        return "<%s for %s> " % (self.__class__.__name__, self.name)

    def mark_as_legal (self, value):
        if value:
            log.log ('restricted', 'mark %s as legal', self.name)
        else:
            log.log ('restricted', 'mark %s as restricted', self.name)
        self.legal = value

    def is_legal (self):
        return self.legal

    def check_legal (self):
        for p in self.plugins:
            if not p.check_legal ():
                self.mark_as_legal (False)
                return self.is_legal ()
        self.mark_as_legal (True)
        return self.is_legal ()

    def get_selected(self):
        return self.selected

    def set_selected(self, desired_status):
        self.selected = desired_status

    def find_compatible_flavors(self, system, arch, distro, core_version,
                                base_version):
        ret = []
        for f in self.flavors:
            if f.is_compatible(system, arch, distro, core_version,
                               base_version):
                ret.append(f)
        return ret

    def get_flavors_for_current_system(self):
        return self.find_compatible_flavors(check.get_os_name (), check.get_arch_name (),
                [check.get_distro_name (), check.get_distro_version ()], check.get_gst_core_version (), 
                check.get_gst_plugins_base_version ())

    def get_best_flavor_for_current_system(self):
        flavors = self.get_flavors_for_current_system()
        if not flavors:
            return None
        else:

      # FIXME: sort by required core version or something

            return flavors[0]

    def get_provided_elements(self):
        """returns the names of the elements in this product"""
        element_names = []
        for plugin in self.plugins:
            for element in plugin.elements:
                element_names.append (element.name)
        return element_names

    def get_provided_decoders(self):
        """returns the caps of the decoders in this product"""
        provided_decoders = []
        for plugin in self.plugins:
            for element in plugin.elements:
                if element.decoders and len(element.decoders) > 0:
                    for caps in element.decoders:
                        provided_decoders.append (caps.caps)
        return provided_decoders

    def get_provided_urisources(self):
        provided_sources = []
        for plugin in self.plugins:
            for element in plugin.elements:
                for caps in element.urisources:
                    provided_sources.append(caps.caps)
        return provided_sources

    def get_installed_status(self):
        """
        returns (num_installed, num_missing)
        checks install status of plugins provided by this product against
        the best available flavour for the current system/setup
        """
        if self.cache_installed_status != None:
            return self.cache_installed_status
        num_installed = 0
        num_missing = 0
        for plugin in self.plugins:
            if plugin.is_installed ():
                num_installed += 1
            else:
                num_missing += 1
        self.cache_installed_status  = (num_installed, num_missing)
        return self.cache_installed_status

    def is_fully_installed(self):
        (num_inst, num_missing) = self.get_installed_status()
        return num_missing == 0

    def is_partially_installed(self):
        (num_inst, num_missing) = self.get_installed_status()
        return num_missing > 0 and num_inst > 0

    def is_partially_or_fully_installed (self):
        (num_inst, num_missing) = self.get_installed_status()
        return num_inst > 0

    def is_nothing_installed (self):
        (num_inst, num_missing) = self.get_installed_status()
        return num_inst == 0

    def updates_available(self):
        """Check if this package can be upgraded"""

        if self.cache_updates_available != None:
            return self.cache_updates_available
        else:
            self.cache_updates_available = False
        if self.is_nothing_installed ():
            return False

        log.debug ('codeina', 'Product (%s) is partially or fully installed' % self.name)

        for plugin in self.get_installed_plugins ():
            log.log ('codeina', 'Checking status for plugin: %s' % plugin.name)
            if plugin.is_newer_than_installed ():
                log.info ('codeina', '%s can be upgraded' % plugin.name)
                self.cache_updates_available = True
                return True
        return False

        """
      # make list of plugins to check (to minimise number of stat() calls)
        plugins = []
        for plugin in self.plugins:
            plugins.append (plugin.name)

        highest_mtime = None
        for element_name in self.get_provided_elements():
            plugin = check.get_plugin_from_element_factory_name (element_name)
            if plugin and not plugin in plugins:
                plugins.append(plugin)

      # check last modification time for each plugin and find the highest
        for plugin in plugins:
            plugin_mtime = check.get_plugin_mtime(plugin)
            if not highest_mtime or plugin_mtime > highest_mtime:
                highest_mtime = plugin_mtime

        flav = self.get_best_flavor_for_current_system()
        if flav and highest_mtime and highest_mtime < flav.last_modified:
            return True
        else:
            return False
        """
  # returns True if self provides all the element other_product provides and
  # other_product provides at least one element that self doesn't contain.

    def is_part_product_of(self, other_product):
        """
        returns True if self provides all the element other_product provides and
        other_product provides at least one element that self doesn't contain.
        """
        elements = self.get_provided_elements()
        other_elements = other_product.get_provided_elements()
        for e in elements:
            if not e in other_elements:
                return False

    # we don't just compare the lengths to guard against duplicate entries

        for e in other_elements:
            if not e in elements:
                return True
        return False

    def provides_element(self, element_name):
        elements = self.get_provided_elements()
        return elements and element_name in elements or False

    def provides_codec(self, provided_caps_list, wanted_caps_string):
        import gst
        wanted_caps = gst.caps_from_string(wanted_caps_string)
        if not wanted_caps:
            ui = __import__(import_ui_string (), [], [], '*')  
            ui.show_error ("Failed to convert string '%s' to GstCaps" % wanted_caps_string)
            return False
        for provided_caps_string in provided_caps_list:
            provided_caps = gst.caps_from_string(provided_caps_string)
            if not provided_caps:
                log.info ('codeina', "Failed to convert string '%s' to GstCaps" % \
                    provided_caps_string)
                continue
            intersect = provided_caps.intersect(wanted_caps)
            if intersect.get_size() > 0:
                return True
        return False

    def provides_decoder(self, caps_string):
        return self.provides_codec(self.get_provided_decoders(),
                                   caps_string)

    def provides_urisource(self, protocol):
        sources = self.get_provided_urisources()
        return sources and protocol in sources or False

    def provides_requirement(self, required_plugin):
        if isinstance (required_plugin, Product):
            return required_plugin == self
        if required_plugin and required_plugin.valid:
            if required_plugin.req_type == 'element':
                return self.provides_element(required_plugin.req_detail)
            if required_plugin.req_type == 'decoder':
                return self.provides_decoder(required_plugin.req_detail)
            if required_plugin.req_type == 'urisource':
                return self.provides_urisource(required_plugin.req_detail)
        log.info ('codeina', "Invalid required plugin object %s" % required_plugin)
        return False

    def flush_cache (self):
        for plugin in self.plugins:
            plugin.flush_cache ()
        self.cache_installed_status = None
        self.cache_updates_available = None

    def get_flavor_distros (self):
        """ 
            get a list of the flavor.distro's that are available for this 
            procuct.
        """
        flavor_distros = []
        for flavor in self.flavors:
            if flavor.distro and not flavor.distro in flavor_distros:
                flavor_distros.append(flavor.distro)
        return flavor_distros

    def get_installed_plugins (self):
        """ get a list of the installed plugins"""
        installed_plugins = []
        for plugin in self.plugins:
            if plugin.is_installed ():
                installed_plugins.append (plugin)
        return installed_plugins

