# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# Written by Tim-Philipp Müller <tim centricular net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from codeina import fxml, check
from codeina.extern.log import log
from codeina.productlist import Product, ProductList, Plugin, Element, ElementType
from codeina.productflavor import ProductFlavor
import time, os

class ProductListParser(fxml.Parser):
    """
    I parse the product list.

    @rtype: L{ProductList}
    """
    def parse(self, handle, id=None):
        """
        Parse from the given file handle.
        The id is used to report problems.
        """
        root = self.getRoot(handle)
        node = root.documentElement

        if node.nodeName != 'products':
            log.warning('productlist', 'No products')
            return

        return self._parse_products(node)

    # parser routines
    def _parse_products(self, node):
        product_list = {}
        products = []
        optional = ('url', 'id', 'logo')
        attrs = self.parseAttributes(node, optional=optional)
        for i in range(0, len(optional)):
            key = optional[i]
            product_list[key] = attrs[i]
        def add_product(p):
            # check if there is already a product with this nick
            for product in products:
                if product.nick == p.nick:
                    log.warning('productlist', 'more than one product with nick %s' % p.nick)
                    return
            products.append(p)

        parsers = { 'product': (self._parse_product, add_product) }
        self.parseFromTable(node, parsers)
        product_list['products'] = products

        return ProductList(**product_list)

    def _parse_product(self, node):
        d = {}
        required = ('nick', 'id', 'logo-uri', 'price', 'rank', 'version')
        optional = ('eula-uri', )
        attrs = self.parseAttributes(node, required=required, optional=optional)
        all = required + optional
        for i in range(0, len(all)):
            if all[i]:
                attr = all[i].replace('-', '_')
                d[attr] = str(attrs[i])
        names = []
        summaries = []
        flavors = []
        plugins = []

        parsers = {
            'name': (self._parse_name, names.append),
            'summary': (self._parse_name, summaries.append),
            'flavor': (self._parse_flavor, flavors.append),
            'plugin': (self._parse_plugins, plugins.append),
        }
        self.parseFromTable(node, parsers)

        # check price
        price_string = d['price']
        price = float(0.0)
        if price_string and price_string[:3] == "EUR":
            price = float(price_string[3:])
        d['price'] = price

        d['rank'] = int(d['rank'])
        d['id'] = int(d['id'])
        d['name'] = self._get_localized(names)
        d['summary'] = self._get_localized(summaries)
        d['flavors'] = flavors
        d['plugins'] = plugins
        product = Product(**d)

        # Select the free-not-fully-installed plugins by default
        if price == 0.0:
            product.selected = not product.is_fully_installed()
        return product

    def _parse_element_type(self, node):
        d = {}
        d['caps'] = node.childNodes[0].data
        return ElementType (**d)

    def _parse_elements (self, node):
        d = {}
        required = ('name',)
        attrs = self.parseAttributes(node, required=required)
        for i in range(0, len(required)):
            key = required[i]
            d[key] = attrs[i]
        decoders = []
        urisources = []
        parsers = {
            'decoder': (self._parse_element_type, decoders.append),
            'urisource': (self._parse_element_type, urisources.append),
        }
        self.parseFromTable(node, parsers)
        if decoders:
               d['decoders'] = decoders
        d['urisources'] = urisources
        elements = Element (**d)
        return elements

    def _parse_plugins(self, node):
        d = {}
        required = ('name', 'version', 'description', 'origin')
        attrs = self.parseAttributes(node, required=required)
        for i in range(0, len(required)):
            key = required[i]
            d[key] = attrs[i]

        elements = []
        parsers = {
            'element': (self._parse_elements, elements.append),
        }
        self.parseFromTable(node, parsers)
        d['elements'] = elements
        plugin = Plugin(**d)
        return plugin

    def _parse_flavor(self, node):
        d = {}
        required = ('system', 'arch', 'distro', 'distro-version',
            'core-req', 'base-req', 'last-modified', 'uri', 'comment')
        optional = ('sha1sum',)
        attrs = self.parseAttributes(node, required=required, optional=optional)
        for i in range(0, len(required)):
            # pesky - -> _ translation for attributes
            key = '_'.join(required[i].split('-'))
            d[key] = attrs[i]

        # create Version objects for the versions
        for k in 'core_req', 'base_req':
            if d[k] is None:
                # use 0.10.0 as the default value, do not bail out on empty versions
                d[k] = "0.10.0"
            d[k] = check.Version(d[k])

        # convert date string to time tuple for 'last-modified'
        d['last_modified'] = time.strptime(d['last_modified'], '%Y-%m-%d')

        flavor = ProductFlavor(**d)
        return flavor

    def _parse_name(self, node):
        # parse a name and its language
        # return tuple of language, name
        lang = 'C'
        if node.attributes:
            for a in node.attributes.items():
                if a[0] == u'xml:lang':
                    lang = a[1]

        return lang, node.childNodes[0].data.strip()

    def _get_localized(self, list, LL=None):
        # from a list of lang, str tuples, pick the one matching our locale
        if not LL:
            language = os.environ.get('LANGUAGE', None)
            if language != None:
                LL = language[:2]
            else:
                lang = os.environ.get('LANG', 'en')
                LL = lang[:2]

        # english is C for now
        if LL == 'en': LL = 'C'

        C = ''
        for lang, text in list:
            if lang == 'C':
                C = text
            if lang == LL:
                return text

        # we can always fall back to C
        return C
