# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

"""

Main User Interface module

This module contains a set of classes with text strings to be shown in dialogs
and notifications windows.

All translatable strings should be stored here, and using gettext:
_("escaping").

"""

from gettext import gettext as _
from codeina.ui.base import *

class NoCodec(Dialog):
    
    """ Display a dialog when an application is missing a codec to decode a file. """

    dialog_title = _("Missing codecs")
    msg = _("You do not have the proper codecs needed to play this file.")

class NoInternet(Notification):

    """ Display a notification when no Internet connection has been found """ 

    dialog_title = _("No connection detected!")
    msg = _("Codeina might not be able to install the \n"
"latest plug-ins.")

class NoProviders(Dialog):

    """ Display a dialog when no provider files have been found """ 

    dialog_title = _("Missing providers")
    msg = _("It seems like you do not have any provider files!")

class NoProducts(Dialog):

    """ Display a dialog when no matching products have been found """ 

    dialog_title = _("Codeina")
    msg = _("No products found to install")

class PackagesInstalled (Dialog):

    """Display a dialog for the installed packages"""

    dialog_title = _("Codeina")
    msg = _("Packages installed:\n")

    def __init__ (self, pkgs):
        self.msg += '\n'.join([ pkg.name for pkg in pkgs])

class ShowMatches(Dialog):

    """ Display a list of available providers
        
    @providers_list = list of tuples ("provider_name", return_code)
    @formatted_text = text to display

    """

    dialog_title = _("Codeina")
    reason = [
_("""Your operating system does not come with the software decoder required to read your media file; either because of legal issues or disk space concerns. You have 2 options to solve this issue:"""),
_("""Use commercial decoding software (codecs), including the necessary licenses to use it safely in any country."""),
_("""Use decoder software based on reverse engineering practices provided by most operating systems in a "do-it-at-your-own-risk" repository. It is then up to you to decide to do so."""),
_("""Click here for more information"""),
_("""Let's look at the MP3 example, one of the most popular music format in the world.

People willing to enjoy their music library need to have a software decoder that supports this file format. MP3 is a technology covered by many patents from various companies. This means that royalties have to be paid for every software solution supporting the MP3 file format.

Many think that the MP3 technology is free, because they usually don't pay for a decoder which is already implemented in their device or PC. This is not true.  People are mislead by the fact that proprietary operating systems - like Microsoft Windows or Apple Mac OS X - include this fee in the overall price you pay when you buy the operating system or the computer. This commercial practice is called "bundling", and the breakdown of the various prices paid is entirely opaque to the end user.""")]

    def __init__ (self, pattern):
        from codeina.providers import CodeinaProvider

        self.pattern = pattern
        self.providers_rank, self.matching_products = \
               CodeinaProvider().get_products_for_pattern(pattern)

    def install(self):
        if not self.matching_products:
            # No products are found for this request
            d = NoProducts()
            d()
            return 1
        for (provider_name, rank) in self.providers_rank:
            if provider_name.lower() == 'fluendo':
                # going to the fluendo shop
                from urllib import quote
                from codeina.configure import CODEINA_SHOP_URL
                from codeina.myutils import build_matches_url
                url = build_matches_url(self.pattern) 
                print "Visit", url
                return 0
            else:
                return 0

    def get_provider_for_response (self, response):
        """
            return the name of the provider for the given response
            None if the response is not known
        """
        for counter in range (len(self.providers_rank)):
            if self.providers_rank[counter][1] == response:
                return self.providers_rank[counter][0]
        return None


class ShowUpdates(Notification):

    """ Display a message when no provider files have been found """ 

    title = _("New Products available!")
    msg = _("Codeina found some updates for:")

    def create_msg (self):
        from codeina import providers
        str = '\n'.join([self.msg] + [ ' - ' + package.name for package in providers.CodeinaProvider ().get_products_with_updates ('fluendo')])
        return str

class FluendoShop(Dialog):

    """ Display a message when no provider files have been found """ 

    dialog_title = _("Codeina Web Shop")
    from codeina.configure import CODEINA_SHOP_URL
    msg = _("visit %s" % CODEINA_SHOP_URL)
